local errors = require('errors')

local _ = package.search('liburoboros') or
    package.searchpath('liburoboros', package.cpath) or
    error('Failed to load liburoboros, check LUA_CPATH')
local role_name = 'replicator'

local RustError = errors.new_class("RustError")

local EXT_FN_NAMES = { 'apply_dml_replace', 'apply_dml_delete', 'apply_ddl', 'test_lib' }

local function init()
    -- luacheck: push ignore
    local module = box.lib.load('liburoboros')
    -- luacheck: pop
    if not module then
        error('Can not load module "liburoboros"')
    end
    local _, err = RustError:pcall(module:load("init"))
    if err then
        return nil, err
    end

    for _, func_name in ipairs(EXT_FN_NAMES) do
        local func = module:load(func_name)
        if not func then
            error('Function ' .. func_name .. ' cannot be loaded from liburoboros')
        end
        rawset(_G, 'liburoboros_' .. func_name, function(...) return func(...) end)
    end
end

local function stop()
end

local function validate_config()
    return true
end

local function apply_config()
    return true
end

local function get_issues()
    local issues = {}
    return issues
end

return {
    role_name = role_name,
    init = init,
    stop = stop,
    validate_config = validate_config,
    apply_config = apply_config,
    get_issues = get_issues,
    dependencies = {
        'cartridge.roles.vshard-storage',
    },
}